/*************************************************************************************
 *
 *    Create an Offscreen GWorld, Randomize and Sort it
 *
 *    GWorld.cp	-	C Source
 *
 *	  Copyright  Apple Computer, Inc. 1988 - 1993
 *	  All rights reserved.
 *
 *    This is the "meat" of the SortPicts program.  This file supplies the
 *	  core functionality to SortPicts by overiding and supplementing
 *	  the WindowObj class.  
 *
 *************************************************************************************/

#include "GWorldObj.h"


Boolean		gSetWTitleOK;		//	This is program wide so that DragHook can get to it!
								//	Is it OK to set the Title?  False during DragWindow!!
pascal	Boolean		AboutBoxFilter( DialogPtr dialog, EventRecord *event, short *itemHit);

#ifndef __powerc
pascal	void		DragHookForThreads( void);
#endif

pascal	void	   *GWorldThreadEntry( void *there);

void				XDrawDragRgn( void);
RgnHandle			gDragRgnHandle;	//	You wouldn't believe me if I told you... but Watch this guy!
GrafPtr				gDragPort;		//	These are used to Hide and Show the Drag Rgn
PenState			gDragPen;		//	Just before a CopyBits action!!  (Can you believe how skanky that is??)
Boolean				gDragRgnHandleValid;	//	Is the DragRgnHandle really a drag RgnHandle?
	
#ifndef __powerc
RgnHandle			ReturnA3( void) = 0x200B;
//						= 0x200B;			//	MOVE.L	A3,D0
#endif


Boolean	GWorldObj :: BeginNewThread( void)
{
	OSErr			errWhatErr;
	
	
	if( gHasThreads)
	{
		HLock( (Handle)this);
		
		errWhatErr = NewThread( kCooperativeThread, 
								GWorldThreadEntry, 
								(void *)this, 
								100000,
								kFPUNotNeeded + kCreateIfNeeded,
								(void**)nil,
								&threadInfo);
		
		if( errWhatErr != noErr)
			return false;
		else
			return true;		//  Successful new thread running!!
	}
	else
	{
		ScramblePict();

		switch( gSortAlgorithm)
		{
			case iShellSort:
				ShellSort();
				break;
			case iHeapSort:
				HeapSort();
				break;
			case iRandomSort:
				if( RandomPixel() & 1)
					HeapSort();
				else
					ShellSort();
				break;
			default:
				ShellSort();
		}
		
		return true;
	}	
}


pascal	void   *GWorldThreadEntry( void *there)		// it's really "this" here
{
	GWorldObj		   *object = (GWorldObj *)there;
	EventRecord			tempEvent;
	
		object->doneSorting = false;
		object->ScramblePict();

//		The following logic is designed to allow the user to intevene if threads doesn't
//		exist.  Note, the following if: if threads is running on the system, just GO!
//		With threads, the user will be able to make menu & window selections all he/she wants.
		if( (object->gHasThreads == false && 
			 EventAvail( mDownMask + keyDownMask, &tempEvent) == false)
		||	(object->gHasThreads == true) )
		{
			switch( object->sortAlgorithm)
			{
				case iShellSort:
					object->ShellSort();
					break;
				case iHeapSort:
					object->HeapSort();
					break;
				default:
					object->ShellSort();
			}
		}

	object->threadInfo = 0;
	HUnlock( (Handle)object);
	return nil;
}


/*************************************************************/
/*                                                           */
/*               A Sort and Draw Help Routine                */
/*                                                           */
/*************************************************************/
void	GWorldObj :: SetSortItem( long index, long data)
{
	short				horiz, vert;
	
	horiz = (short)(index % pictWidth);
	vert = (short)(index / pictWidth);
	
	copyBitsRect.left = 0;
	copyBitsRect.right = pictWidth;
	
	if( vert >= copyBitsRect.bottom)
		copyBitsRect.bottom = vert+1;

	if( vert < copyBitsRect.top)
		copyBitsRect.top = vert;

	sortListPtr[index] = data;
	offset = horiz + (vert * pictRowBytes);
	offscreenPtr[offset] = (Byte)data;
}

void	GWorldObj :: ExchangeSortItem( long one, long theOther)
{
	long		temp;
	
	temp = sortListPtr[one];
	SetSortItem( one, sortListPtr[theOther]);
	SetSortItem( theOther, temp);
}


OSErr	GWorldObj :: YieldIfTime( long *yieldTime)
{
	EventRecord		tempEvent;
	long			*tickPtr = (long *)0x16A;

	if( *tickPtr >= *yieldTime )
	{
		UnlockPixels( sortGWorld->portPixMap);
		GetPort( &savePort);
		SetPort( me);
		DrawObj();
		SetPort( savePort);

		if( gHasThreads == true) 
			if( yieldOnlyToMe == false)
				YieldToAnyThread();
			else
				YieldToThread( kApplicationThreadID);
		else		//	No Threads package
			if( EventAvail( mDownMask + keyDownMask, &tempEvent) == true)
			{
				if( tempEvent.what == mouseDown)
					GetNextEvent( mDownMask + keyDownMask, &tempEvent);
				return memLockedErr;
			}
		if( LockPixels( sortGWorld->portPixMap) == false)
			return userCanceledErr;
		offscreenPtr = (Byte *)GetPixBaseAddr(  sortGWorld->portPixMap);
		*yieldTime = *tickPtr + gYieldDelay;
	}
	return noErr;
}

long	GWorldObj :: CheckSumSortData( void)
{
	long		loop;
	long		cs = 0;
	
	for( loop = 0; loop < N; ++loop)
		cs += sortListPtr[loop];
	
	return cs;

}

long	GWorldObj :: CheckSumPixelData( void)
{
	long		loop;
	long		cs = 0;
	long		size = pictRowBytes * pictHeight;
	
	for( loop = 0; loop < size; ++loop)
		cs += (long)offscreenPtr[loop];
	
	return cs;

}


void	GWorldObj :: UpdateObj( void)
{
	Rect		tempRect = copyBitsRect;
	Rect		windowRect = windPictRect;
	
	copyBitsRect = worldPictRect;
	DrawObj();
	copyBitsRect = tempRect;
}

void	GWorldObj::DrawObj( void)
{
	Rect		worldRect = copyBitsRect;
	Rect		windowRect = windPictRect;
	
	XDrawDragRgn();

	SetPort( me);
	PenNormal();

	if( (gSetWTitleOK & needToUpdateWindowTitle) == true)
	{
		SetWTitle( me, windowTitle);
		needToUpdateWindowTitle = false;
	}
	
	if( randomizing == false)
	{
		if( gEraseWindow == true)
			EraseRect( &windowRect);
		
		if( gQualudesUpdate == false)
		{		//	This code sets CopyBits to do the right thing...
			SetRect( &windowRect, windPictRect.left, 
								  windPictRect.top + worldRect.top,
								  windPictRect.left + pictWidth,
								  windPictRect.top + worldRect.bottom);
		}	//	The Qualudes option tells copybits to do screwy things
		
		CopyBits(	(const BitMap *) (*(sortGWorld->portPixMap)),
					(const BitMap *) (*(((CWindowPtr)me)->portPixMap)),
					&worldRect, &windowRect, srcCopy, (RgnHandle)0);
		ValidRect( &windowRect);
		
		if( doneSorting == false)
			//		Invert the top & bottom rect so that no drawing takes place!!
			//	The sort algorithm expands this rect so that it is the smallest
			//	that it needs to be to incorporate the bits modified
			SetRect( &copyBitsRect, 0, 0x7FFF, pictWidth, -1);
		else
			copyBitsRect = worldPictRect;
	}
	else
	{
		short				barWidth;
		short				barHeight;
		short				barLeftEdge;
		short				barTopEdge;
		Rect				barRect;
		PixPatHandle		barPixPat = GetPixPat( 128);
		
		ValidRect( &windowRect);
		if( pictWidth >= 100)
			barWidth = 100;
		else
			barWidth = pictWidth - 10;
		barLeftEdge =  windPictRect.left + ((pictWidth-barWidth) >> 1);
		if( pictHeight >= 20)
			barHeight = 12;
		else
			barHeight = pictHeight;
		
		if( pictHeight >= 100)
			barTopEdge = windPictRect.top + pictHeight-60;
		else
			barTopEdge = windPictRect.top + ((pictHeight-barHeight) >> 1);
		
		
		SetRect( &barRect, barLeftEdge, barTopEdge, barLeftEdge+barWidth, barTopEdge+barHeight);
		InsetRect( &barRect, -1, -1);
		FrameRect( &barRect);
		InsetRect( &barRect, 1, 1);
		barRect.right = barLeftEdge + (barWidth * percentComplete / 100);
		if( barPixPat)
		{
			FillCRect( &barRect, (PixPatHandle) barPixPat);
			DisposePixPat( barPixPat);
		}
		else
			PaintRect( &barRect);
		
		//	Erase the right side of the rectangle
		if( percentComplete == 5)
		{
			barRect.left = barRect.right+1;
			barRect.right = barLeftEdge+barWidth;
			EraseRect( &barRect);
		}
	}

	XDrawDragRgn();
}


void	GWorldObj :: BackGroundObj( void)
{
	if( gHasThreads == true) 
		YieldToThread( threadInfo);

	if( (needToUpdateWindowTitle & gSetWTitleOK) == true)
		DrawObj();

	if( doneSorting == true && gContinuous == true && TickCount() > (5 * 60 + finishedTime))
	{
		MouseClickObj( (EventRecord *) 0);
	}
}	


void	GWorldObj :: IdleObj( void)
{
	WindowPeek		   *firstWindow = (WindowPeek *) 0x9D6;		//	Start at the first window
	WindowPeek			windowList = (WindowPeek) *firstWindow;	//	Walk the list
	GWorldObj		   *target;
	short				windowCount;
	
	windowCount = 1;
	while( windowList)
	{
		target = (GWorldObj *)(windowList->refCon);
		if( windowCount++ == gNextBkgndWindowNum)
		{
			target->BackGroundObj();
			++gNextBkgndWindowNum;
			return;
		}
		windowList = windowList->nextWindow;
	}
	//	If we get here we obviously have a gNextBkgndWindowNum which is > than the number
	//	of open windows.  Therefore, set gNextBkgnd to 1 and send an update event to the
	//	first window in the window list (if there is a first window)
	gNextBkgndWindowNum = 1;
	windowList = (WindowPeek) *firstWindow;
	if( windowList)
	{
		target = (GWorldObj *)(windowList->refCon);
		target->BackGroundObj();
		++gNextBkgndWindowNum;
	}
}




/****************************************************************************/
/*                                                                          */
/*                              Scramble Pict                               */
/*                                                                          */
/****************************************************************************/
void	GWorldObj::ScramblePict( void)
{
	long			loop;
	unsigned long	index;
	RGBColor		pixColor = {0, 0, 0};
	Byte			pixelData;
	short			horiz, vert;
	short			indexLength;
	long			oneTwentieth;
	long			loopTwentieth;
	short			percentPerPiece = 5;
	EventRecord		tempEvent;
	long		   *tickPtr = (long *)0x16A;
	
		
	if( LockPixels( sortGWorld->portPixMap) == true)
	{
		randomizing = true;
		percentComplete = 0;
		SetTitle( (const unsigned char*)"\pRandomizing");
		GetPort( &savePort);
		ClearAllBits();
		SetPort( me);
		pixColor.red = pixColor.blue = pixColor.green = 0;
		DrawObj();
		
		offscreenPtr = (Byte *)GetPixBaseAddr(  sortGWorld->portPixMap);
		HLock( (Handle)usedPixels);
		usedPixelsPtr = *usedPixels;
//		if( !usedPixelsPtr)
//			DebugStr("\pScramble: usedPixelsPtr == nil!");
		
		HLock( (Handle)sortList);
		sortListPtr = *sortList;
//		if( !sortListPtr)
//			DebugStr("\pScramble: sortListPtr == nil!");

		horiz = 0;
		vert = 0;
		offset = 0;
		RandomInit( N);
		oneTwentieth = (N / 20) + 0;
		loopTwentieth = 0;
	
		for( loop = 0; loop < N; ++loop)
		{
			index = RandomPixel();
			indexLength = 0;
			while( usedPixelsPtr[index >> 3] & (128 >> (index & 7)) )
			{
				++index;
				if( index == N)
					index = 0;
				else
					if( ++indexLength == (pictWidth >> 4))
					{
						indexLength = 0;
						index = RandomPixel();
					}
			}

			usedPixelsPtr[index >> 3] |= (Byte) (128 >> (index & 7));
			
			pixelData = offscreenPtr[horiz + offset];	// This fetches the pixel
			
			sortListPtr[index] = (loop << 8) + pixelData;	// Put the pixel and its # into the scrambled list

			++loopTwentieth;
			++horiz;
			if( horiz == pictWidth)
			{
				++vert;
				horiz = 0;
				offset += pictRowBytes;
			}
//			SetCPixel( (short)(index % pictWidth), (short)(index / pictWidth), &pixColor);

			if( loopTwentieth == oneTwentieth)
			{
				UnlockPixels( sortGWorld->portPixMap);

				GetPort( &savePort);
				SetPort( me);
				percentComplete += percentPerPiece;
				if( percentComplete == 95)
				{
					oneTwentieth /= 5;
					percentPerPiece = 1;
				}
				loopTwentieth = 0;
				DrawObj();
				SetPort( savePort);

				if( gHasThreads == true) 
					if( yieldOnlyToMe == false)
						YieldToAnyThread();
					else
						YieldToThread( kApplicationThreadID);
				else		//	No Threads package
					if( EventAvail( mDownMask + keyDownMask, &tempEvent) == true)
					{
						if( tempEvent.what == mouseDown)
							GetNextEvent( mDownMask + keyDownMask, &tempEvent);
						loop = N;
					}

				if( LockPixels( sortGWorld->portPixMap) == false)
//					DebugStr("\pScramble:  Can't lock the pixels back down!!");
					goto ScrambleErrQuit;
				offscreenPtr = (Byte *)GetPixBaseAddr(  sortGWorld->portPixMap);
				yieldTime = TickCount() + gYieldDelay;
			}
		}
		

//	SetTitle( "\pReconstructing Random Image");
	offscreenPtr = (Byte *)GetPixBaseAddr(  sortGWorld->portPixMap);
	horiz = 0;
	for( loop = 0; loop < N; ++loop)
	{
		pixelData = (Byte)(sortListPtr[loop] & 0xFF);
		offscreenPtr[horiz] = pixelData;
		++horiz;
		if( horiz == pictWidth)
		{
			horiz = 0;
			offscreenPtr += pictRowBytes;
		}
	}
	
	UnlockPixels( sortGWorld->portPixMap);

ScrambleErrQuit:
	HUnlock( (Handle)usedPixels);
//	DisposeHandle( (Handle)usedPixels);
	HUnlock( (Handle)sortList);
	randomizing = false;
	SetTitle( (const unsigned char*)"\pDone Randomizing");
	
	DrawObj();


	}

//	SwapMMUMode ( &memoryMode);		// Return to whatever mode...

}





/****************************************************************************/
/*                                                                          */
/*                        Randomization Stuff                               */
/*                                                                          */
/****************************************************************************/
void	GWorldObj :: RandomInit( long max)
{
	long			tempN;
	short			numNBits;
	
//	Random Number Initialization
	tempN = max;
	numNBits = 1;				// Using the additive conguential method (Random numbers)
	while (tempN >>= 1)			// Count the number of bits required to hold N	
		++numNBits;				//	example:  to hold 0x37 requires 6 bits (0-3F)

	numRandomBits = 31;	
	numShiftBits = numRandomBits - numNBits;

	randomSeed = TickCount();
//	randomSeed = 0x14E0F;		//	TickCount();	This "PATCH" keeps the Random #
								//	generator from ever getting a bad seed value!
	maxRandom = max;
}



long	GWorldObj :: RandomPixel( void)
{
	long		randomNum = 0x7FFFFFFF;		//	The ultimate long
	
	while( randomNum >= maxRandom)
	{
//		RandomNum generator
		randomNum = randomSeed = (randomSeed & 1) ^ ((randomSeed & (1 << 28)) >> 28)
							   ? (randomSeed >> 1) | 0x40000000 : randomSeed >> 1;
//		This next line "mixes" some of the upper bits with the lower bits
		randomNum = (randomNum >> numShiftBits) ^ (randomNum & (0x7FFFFFFF >> numShiftBits));
	}
	
	return randomNum;	
}










/****************************************************************************/
/*                                                                          */
/*                           Dumb Application                               */
/*                                                                          */
/****************************************************************************/

short		GWorldObj :: gSortSize;
Boolean		GWorldObj :: gEraseWindow;
Boolean		GWorldObj :: gHasThreads;
Boolean		GWorldObj :: gQualudesUpdate;
Boolean		GWorldObj :: gContinuous;
Boolean		GWorldObj :: gSaveWindowPositions;
short		GWorldObj :: gYieldDelay;
short		GWorldObj :: gNumPictures;
MenuHandle	GWorldObj :: gPicturesMenu;
short		GWorldObj :: gWindOffSet;
short		GWorldObj :: gNextBkgndWindowNum;
short		GWorldObj :: gSortAlgorithm;

void	GWorldObj::InitObj( void)
{
	long	threadsPresentBit;
	
//	inherited :: InitObj();		// This DOESN'T WORK IN MPW C++ ????
	
	isColor = gMac.hasColorQD;		// Retain the color bit
	useActivateClicks = false;		// Do not interpret activate events as action events

//	Set GWorldObj specific global variables
	gEraseWindow = false;
	gYieldDelay = 6;
	gSortSize = 0;
	gWindOffSet = 0;
	gSetWTitleOK = true;
	gDragRgnHandle = (RgnHandle) 0;
	gDragRgnHandleValid = false;
	gQualudesUpdate = false;
	gContinuous = false;
	gSaveWindowPositions = false;
	gNextBkgndWindowNum = 1;
	gSortAlgorithm = iHeapSort;	//	default is shell sort
	
	if( Gestalt( gestaltThreadMgrAttr, &threadsPresentBit) == noErr)
		gHasThreads = (1<<gestaltThreadMgrPresent) & threadsPresentBit ? true : false;
	
//	Build the Pictures Menu #PICTURESMENU
	gPicturesMenu = NewMenu( PICTURESMENU, (const unsigned char*)"\pPictures");
	AddResMenu( gPicturesMenu, 'PICT');
	InsertMenu( gPicturesMenu, -1);

//	Count the number of Pictures in the menu
	gNumPictures = CountMItems( gPicturesMenu);
	
	if( gHasThreads)
	{
		long			   *DragHook = (long *)0x09F6;		// DragHook is called during DragWindow, etc.
		Handle				startupPictStrHandle;

#ifndef __powerc
		*DragHook = (long) DragHookForThreads;		//	Patch DragHook
#endif

		startupPictStrHandle = GetNamedResource( 'STR#', (const unsigned char*)"\pStartup Pictures");
		if( startupPictStrHandle)	// If there is a predefined list of startup pictures
		{							//	Load them in now
			short				startupPictID;
			ResType				tempType;
			Str255				tempName;
			Boolean				newWindow = true;
			short				windowCount = 1;
			GWorldObj		   *newObj;
			
			GetResInfo( startupPictStrHandle, &startupPictID, &tempType, tempName);
			while( newWindow == true)
			{
				newWindow = false;			//	Unless this window works, stop making them
				newObj = new GWorldObj;
				if( newObj)
				{
					GetIndString( newObj->pictureName, startupPictID, windowCount);
					if( newObj->pictureName[0])
					{
						newWindow = newObj->NewWindowObj();	// did this window work?
					}
					++windowCount;
					YieldToAnyThread();
				}
			}
			
		}
		else		//	If there is no predefined list of pictures, 
		{			//	just load in consecutive ones
			GWorldObj		   *a;
			Boolean				newWindow = false;
			short				windowCount;

			windowCount = 1;
			while( newWindow == true)
			{
				a = new GWorldObj;
				if( a)
				{
					GetItem( gPicturesMenu, windowCount, a->pictureName);
					newWindow = a->NewWindowObj();
					YieldToAnyThread();
	//	RST:  The following fix will force only one window to be created!
					newWindow = false;
				}
				++windowCount;
			}
		}
	}
	else		//	gHasThreads == NOT!!
	{
		GWorldObj		   *a;

		a = new GWorldObj;
		if( a)
		{

			GetItem( gPicturesMenu, 1, a->pictureName);
			a->NewWindowObj();
		}
	}
}

Boolean	GWorldObj::PrepareGWorld( void)
{
	Rect			worldRect;
	OSErr			myOSErr;
	GWorldPtr		myGWorld;
	GDHandle		oldGDevice;
	CGrafPtr		oldGWorld;

	sortPict = (PicHandle)GetNamedResource( 'PICT', pictureName);
	
	if( sortPict == nil)
		return false;
	
	randomizing = true;
	percentComplete = 0;
	doneSorting = false;
	yieldOnlyToMe = false;
	
	worldPictRect = (*sortPict)->picFrame;
	pictWidth = worldPictRect.right - worldPictRect.left;
	pictHeight = worldPictRect.bottom - worldPictRect.top;
	
	SetRect(   &worldPictRect, 0, 0, pictWidth, pictHeight);
	copyBitsRect = worldPictRect;
	windPictRect = worldPictRect;
	worldRect = worldPictRect;

	GetGWorld( &oldGWorld, &oldGDevice);

	myOSErr = NewGWorld( &myGWorld, 8, &worldRect, (CTabHandle) 0, (GDHandle)0, 0);
	
	if( myOSErr != noErr)
	{
		ReleaseResource( (Handle)sortPict);
		return false;
	}
			
	sortGWorld = myGWorld;
	
	pictRowBytes = (*GetGWorldPixMap( sortGWorld))->rowBytes & 0x3FFF;
	sortListSize = (long) pictWidth * (long) pictHeight ;
	
	sortList = (long **) NewHandle( sortListSize * sizeof(long) );
				
	if( sortList == nil)
	{
		ReleaseResource( (Handle)sortPict);
		DisposeGWorld( myGWorld);
		return false;
	}

	usedPixels = (Byte **) NewHandle( (sortListSize >> 3) + 1);
	
	if( usedPixels == nil)
	{
		ReleaseResource( (Handle)sortPict);
		DisposeGWorld( myGWorld);
		DisposeHandle( (Handle)sortList);
		return false;
	}
	
	GetGWorld( &oldGWorld, &oldGDevice);
	SetGWorld( sortGWorld, (GDHandle)0);
	
	EraseRect( &worldRect);
	DrawPicture( sortPict, &worldRect);
	
	ReleaseResource( (Handle)sortPict);

	SetGWorld( oldGWorld, oldGDevice);
	
	return true;			//	YES!! The everything worked out!
}

Boolean	GWorldObj::NewWindowObj( void)
{
	Rect		winRect;
	Handle		winRectHandle;
	
	if( PrepareGWorld() == true)
	{
		aboutBox = false;
		windPictRect = worldPictRect;
		winRect = windPictRect;
		sortAlgorithm = gSortAlgorithm;
		
		OffsetRect( &winRect, 40 + gWindOffSet, 40 + gWindOffSet);
//		InsetRect( &windPictRect, -10, -10);
		gWindOffSet += 40;
		if( gWindOffSet >= 300)
			gWindOffSet = 0;
		
		winRectHandle = GetNamedResource( 'RECT', pictureName);
		if( winRectHandle)
			winRect = **((Rect**)winRectHandle);

		me = NewCWindow( NIL, &winRect, (ConstStr255Param)"\pLoading", true, noGrowDocProc, (WindowPtr)-1, true, 
			/* CRITICALLY IMPORTANT:: */   (long) this);
		
		if( winRectHandle)
			ReleaseResource( winRectHandle);
		
		if( me)		/* Do these things if the window was created ! */
		{
			((WindowPeek)me)->windowKind = WindowObjKind;

			if( BeginNewThread() == true)
			{
				return true;
			}
			DisposeWindow( me);
			DisposeGWorld( sortGWorld);
			DisposeHandle( (Handle)sortList);
			DisposeHandle( (Handle)usedPixels);
			return false;
		}
		else			//  me == nil		
			return false;
	}
	else
		return false;
}


Boolean	GWorldObj::AboutObj( void)
{
	char	  **pictName = (char **)GetResource( 'STR ', ABOUTPICTURENAMEID);
	short		loop;
	short		itemType;
	Handle		itemHandle;
	Rect		itemRect;
	short		itemHit;
	
	if( pictName)
	{
		//	Copy the picture name string from the Handle to this->pictureName
		for( loop = pictureName[0] = (*pictName)[0]; loop ; --loop)
			pictureName[loop] = (*pictName)[loop]; 
		
		ReleaseResource( (Handle)pictName);
		
		if( PrepareGWorld() == true)
		{
			aboutBox = true;
			me = (WindowPtr) GetNewDialog( ABOUTDIALOG, (Ptr) 0, (WindowPtr) -1);
			if( me)
			{
				((WindowPeek)me)->refCon = (long)this;
				GetDItem( (DialogPtr) me, ABOUTDIALOGPICT, &itemType, &itemHandle, &itemRect);
				windPictRect = itemRect;
				
				if( gHasThreads)
				{
					if( BeginNewThread() == true)
					{
#ifdef __powerc
						AboutBoxFilterUPP = NewModalFilterProc( (long (*)(void))AboutBoxFilter);
#endif			
						itemHit = -1;
						while( itemHit != 1)
						{
#ifdef __powerc
							ModalDialog( AboutBoxFilterUPP, &itemHit);
#else
							ModalDialog( AboutBoxFilter, &itemHit);
#endif
						}
						
						if( threadInfo != 0)
							DisposeThread( threadInfo, 0, false);
#ifdef __powerc
						if( AboutBoxFilterUPP)
							DisposePtr( (Ptr) AboutBoxFilterUPP);
#endif
					}
				}
				else
				{
					itemHit = -1;
					
					//	First, allow the redraw algorithm to kick in
					for( loop = 0; loop < 3 && itemHit == -1; loop ++)
#ifdef __powerc
						ModalDialog( (class RoutineDescriptor*)0, &itemHit);
#else
						ModalDialog( (ModalFilterProcPtr)0, &itemHit);
#endif

					//	Randomize and Sort the picture
					if( itemHit == -1)
						BeginNewThread();
					
					//	Now wait for the user to quit this dialog
					while( itemHit != 1)
#ifdef __powerc
						ModalDialog( (class RoutineDescriptor*)0, &itemHit);
#else
						ModalDialog( (ModalFilterProcPtr)0, &itemHit);
#endif
				}
	
				DisposeDialog( (DialogPtr) me);
				me = (WindowPtr)0;
			}
	
			DisposeHandle( (Handle)sortList);
			DisposeHandle( (Handle)usedPixels);
			DisposeGWorld( sortGWorld);
			return false;
		}
		else
		{
			return false;
		}
	}
	return false;
}

pascal Boolean	AboutBoxFilter( DialogPtr dialog, EventRecord *event, short *itemHit)
{
	GWorldObj		   *object = (GWorldObj *)((WindowPeek)dialog)->refCon;
	GWorldObj		   *target;
	WindowPtr			whichWindow;
	short				windowPart;
	
	if( (event->what == keyDown) && 
	   ( ((event->message & charCodeMask) == 13) || 	//	Test for RETURN key
	     ((event->message & charCodeMask) == 3 ) ) )	//	Test for ENTER key
	{
		*itemHit = 1;
		return true;
	}
	
	if( event->what == mouseDown)
	{
		windowPart = FindWindow( event->where, &whichWindow);
		if( whichWindow == object->me && windowPart == inDrag)
		{
			object->DragWindowObj( event);
			return true;
		}
	}

	if( event->what == updateEvt)
	{
		target = (GWorldObj *)((WindowPeek)event->message)->refCon;

		target->UpdateObj();
		if( target == object)
			return false;
		return true;
	}

	if( event->what == nullEvent)
		object->IdleObj();

	return false;
}



void	GWorldObj::AdjustMenusObj( void)
{
	Str255				yieldDelayStr;
	Str255				yieldDelayMenuStr;
	MenuHandle			userOptionsMenu = GetMHandle( USEROPTIONSMENU);
	MenuHandle			sortAlgorithmMenu = GetMHandle( mSortAlgorithm);
	short				yieldDMLen, numStrLen;
	MenuHandle	menu;

	menu = GetMHandle( mFile);			// Do the File menu first
	if( menu)
	{
		EnableItem(menu, iNew);				// Enable the items, one at a time...
		EnableItem(menu, iOpen);
		if( me != NIL)
		{
			EnableItem(menu, iClose);
		}
		else
		{
			DisableItem(menu, iClose);
		}
	
		DisableItem(menu, iSave);
		DisableItem(menu, iSaveAs);
		DisableItem(menu, iRevert);
		DisableItem(menu, iPageSetup);
		DisableItem(menu, iPrint);
		EnableItem(menu, iQuit);
	}
	
	menu = GetMHandle( mEdit);
	if( menu)
	{
		DisableItem(menu, iUndo);
		DisableItem(menu, iCut);
		DisableItem(menu, iCopy);
		DisableItem(menu, iClear);
		DisableItem(menu, iPaste);
	}

	if( userOptionsMenu)
	{
		NumToString( gYieldDelay, yieldDelayStr);
		GetItem( userOptionsMenu, UO_SHOWTTYDELAY, yieldDelayMenuStr);
		yieldDMLen = yieldDelayMenuStr[0];
		while( yieldDelayMenuStr[yieldDMLen] != ' ')
			yieldDMLen--;
		
		numStrLen = yieldDelayStr[0];
		yieldDelayMenuStr[0] = yieldDMLen + numStrLen;
		while( numStrLen)
		{
			yieldDelayMenuStr[yieldDMLen + numStrLen] = yieldDelayStr[numStrLen];
			numStrLen--;
		}
		SetItem( userOptionsMenu, UO_SHOWTTYDELAY, yieldDelayMenuStr);
		CheckItem( userOptionsMenu, UO_ERASE, gEraseWindow);
		CheckItem( userOptionsMenu, UO_QUALUDES, gQualudesUpdate);
		CheckItem( userOptionsMenu, UO_CONTINUOUS, gContinuous);
		CheckItem( userOptionsMenu, UO_SAVEWINDPOS, gSaveWindowPositions);
		CheckItem( userOptionsMenu, UO_SORT_0, gSortSize == 0);
		CheckItem( userOptionsMenu, UO_SORT_1, gSortSize == 1);
	}

	if( sortAlgorithmMenu)
	{
		CheckItem( sortAlgorithmMenu, iShellSort, gSortAlgorithm == iShellSort);
		CheckItem( sortAlgorithmMenu, iHeapSort , gSortAlgorithm == iHeapSort );
		CheckItem( sortAlgorithmMenu, iRandomSort , gSortAlgorithm == iRandomSort );
	}
}





void	GWorldObj::MenuObj( short menuID, short menuItem, EventRecord *event)
{
	WINDOWOBJ	   *a;
	Str255			daName;
	
	switch( menuID )
	{
		case mApple:
			if( menuItem == iAbout)
			{
				WINDOWOBJ		   *a;
				a = new WINDOWOBJ;
				if( a)
				{
					a->AboutObj();
					delete a;
				}
			}
			else		//	Open a desk accessory...
			{
				GetItem(GetMHandle(mApple), menuItem, daName);
				(void) OpenDeskAcc(daName);
			}
			break;
		case mFile:
			switch( menuItem)
			{
				case iClose:
					CloseWindowObj();
					break;
				case iPageSetup:
					PageSetupObj();
					break;
				case iPrint:
					PrintObj();
					break;
				default:
					AppMenu( menuID, menuItem);		// This is like inheritance!!
					break;
			}
			break;

		case USEROPTIONSMENU:
			switch( menuItem)
			{
				case UO_ERASE:		//	Erase Window User Option
					gEraseWindow = !gEraseWindow;
					break;
				case UO_QUALUDES:	//	Alter the CopyBits behavior
					gQualudesUpdate = !gQualudesUpdate;
					break;
				case UO_CONTINUOUS:	//	Run for ever
					gContinuous = !gContinuous;
					break;
				case UO_SAVEWINDPOS://	Run for ever
					gSaveWindowPositions = !gSaveWindowPositions;
					break;
				case UO_SORT_0:		//	Complete the Sort 100%
					gSortSize = 0;
					break;
				case UO_SORT_1:		//	Finish 1 step before the sort is complete
					gSortSize = 1;
					break;
				case UO_BLANK:		//	This is a dividing line
					break;
				case UO_SHOWTTYDELAY:		//	This displays the Time To Yield
					break;
				case UO_DECREASETTY:		//	Decrease Time To Yield Delay
					if( gYieldDelay > 5)
						gYieldDelay -= 4;
					break;
				case UO_INCREASETTY:		//	Increase Time To Yield Delay
					gYieldDelay += 4;
					break;
			}
			break;

		case PICTURESMENU:
			a = new WINDOWOBJ;
			if( a)
			{
				HLock( (Handle) a);
				GetItem( gPicturesMenu, menuItem, a->pictureName);
				HUnlock( (Handle)a);
				if( a->NewWindowObj() == false)
					delete a;
			}
			break;
			
		case mSortAlgorithm:
			gSortAlgorithm = menuItem;
			break;
		
		default:
			if( (event->message & charCodeMask) == 'n')
			{
				WINDOWOBJ		   *a;
				a = new WINDOWOBJ;
				if( a)
				{
					MenuHandle	gPicturesMenu = GetMHandle( PICTURESMENU);
					
					if( gPicturesMenu)
					{
						GetItem( gPicturesMenu, (short)(((short)(TickCount()) & 0x7FFF) % a->gNumPictures), a->pictureName);
						if( a->NewWindowObj() == false)
							delete a;
					}
				}
			}
			else if( ((event->message & charCodeMask) == ',') || 
					 ((event->message & charCodeMask) == '<') )
			{
							if( a->gYieldDelay > 5)
									a->gYieldDelay -= 4;
			}
			else if( ((event->message & charCodeMask) == '.') || 
					 ((event->message & charCodeMask) == '>') )
							gYieldDelay += 4;
			else if( ((event->message & charCodeMask) >= '1') && 
					 ((event->message & charCodeMask) <= '0') )
			{
							WINDOWOBJ		   *a;
							a = new WINDOWOBJ;
							if( a)
							{
								MenuHandle	gPicturesMenu = GetMHandle( PICTURESMENU);
								
								if( gPicturesMenu)
								{
									GetItem( gPicturesMenu, (short)(event->message & charCodeMask) - '0', a->pictureName);
									if( a->NewWindowObj() == false)
										delete a;
								}
							}
			}
			else
				AppMenu( menuID, menuItem);				// This is like inheritance!!
			break;
	}
	HiliteMenu(0);
}




void	GWorldObj::GrowWindowObj( EventRecord *event)
{
	//		This window doesn't grow...
	int			a = 0;
	if( a != 0)
		event->modifiers = 0;
}

void	GWorldObj::MouseClickObj( EventRecord *event)
{
	GDHandle		oldGDevice;
	CGrafPtr		oldGWorld;
	Rect			worldRect, windRect;
	EventRecord	   *cpEvent = event;		//	Gets CFront off my back!

	if( doneSorting == true)
	{
		randomizing = true;
		percentComplete = 0;
		doneSorting = false;

		sortPict = (PicHandle)GetNamedResource( 'PICT', pictureName);
		
		if( sortPict)
		{
			SetPort( me);
			worldRect = worldPictRect;
			windRect = windPictRect;
			EraseRect( &windRect);

			GetGWorld( &oldGWorld, &oldGDevice);
			SetGWorld( sortGWorld, (GDHandle)0);

			EraseRect( &worldRect);
			DrawPicture( sortPict, &worldRect);
			
			ReleaseResource( (Handle)sortPict);
	
			SetGWorld( oldGWorld, oldGDevice);

			BeginNewThread();
		}
	}
	else
	{
		Str255			oldWindowTitle;
		Str255			tempTitle;
		Str255			turboTitle = "\pTurbo";
		Boolean			equal;
		short			loop;
		
		GetWTitle( me, oldWindowTitle);
		SetTitle( turboTitle);
		yieldOnlyToMe = true;
		
		while( StillDown() && (doneSorting == false))
		{
			YieldToThread( threadInfo);
			GetWTitle( me, tempTitle);
			equal = true;
			for( loop = 0; (loop <= tempTitle[0]) && (equal == true); ++loop)
				equal = (tempTitle[loop] == turboTitle[loop]);
			if( equal == false)
			{
					GetWTitle( me, oldWindowTitle);
					SetTitle( turboTitle);
			}
		}
			
		yieldOnlyToMe = false;
		SetTitle( oldWindowTitle);
	}
}

Boolean	GWorldObj::CloseWindowObj( void)
{
	if( me == FrontWindow())
	{
		if( gHasThreads && threadInfo != 0)
			DisposeThread( threadInfo, 0, false);
		DisposeHandle( (Handle)sortList);
		DisposeHandle( (Handle)usedPixels);
		DisposeGWorld( sortGWorld);
		DisposeWindow( me);					//  Call the Mac to eliminate the window!

		return true;
	}
	else
		return false;
		
}

void	GWorldObj::DrawGrowIconObj( void)
{
	//	Don't draw the grow thing!
}


void	GWorldObj::ClearAllBits( void)
{
	long		loop;
	long		sortListBytes;
	
	HLock( (Handle)usedPixels);
	usedPixelsPtr = *usedPixels;
	
	sortListBytes = (N >> 3) +1;
	for(loop = 0; loop < sortListBytes; ++loop)
		usedPixelsPtr[loop] = 0;
	
	HUnlock( (Handle)usedPixels);
}

void	GWorldObj::SetBit( long bitNum)
{
	long				byteIndex;
	unsigned char		bitIndex;
	
	HLock( (Handle)usedPixels);
	usedPixelsPtr = *usedPixels;
	
	byteIndex = bitNum >> 3;
	bitIndex = (unsigned char) (128 >> (bitNum & 7));
	
	usedPixelsPtr[byteIndex] |= bitIndex;
	HUnlock( (Handle)usedPixels);
}

Boolean	GWorldObj::TestBit( long bitNum)
{
	long				byteIndex;
	unsigned char		bitIndex;
	unsigned char		bitValue;
	
	HLock( (Handle)usedPixels);
	usedPixelsPtr = *usedPixels;
	
	byteIndex = bitNum >> 3;
	bitIndex = (unsigned char) (128 >> (bitNum & 7));
	
	bitValue = usedPixelsPtr[byteIndex] & bitIndex;
	
	HUnlock( (Handle)usedPixels);
	
	if( bitValue == 0)
		return false;
	else
		return true;
}

void	GWorldObj::ClearBit( long bitNum)
{
	long				byteIndex;
	unsigned char		bitIndex;
	
	HLock( (Handle)usedPixels);
	usedPixelsPtr = *usedPixels;
	
	byteIndex = bitNum >> 3;
	bitIndex = (unsigned char) (255- (128 >> (bitNum & 7)));
	
	usedPixelsPtr[byteIndex] &= bitIndex;
	HUnlock( (Handle)usedPixels);
}



void	GWorldObj :: DragWindowObj( EventRecord *event)
{
	Rect			globalWindRect;			//	global as in desk top, not program wise
	Point		   *tempPoint;
	Handle			rsrcRect;
	GrafPtr			savedPort;
	
	gDragRgnHandleValid = true;
	
	WindowObj :: DragWindowObj( event);
	
	gDragRgnHandleValid = false;
	
	if( gSaveWindowPositions == true)
	{
	//	Create a RECT within the Actual application which is the RECT of the Window's
	//	Global portRect.  OK?  But remove this code before it ships (because it's sick
	//	to ship code which modifies its own file!!).
		
		GetPort( &savedPort);
		SetPort( me);
		
		globalWindRect = me->portRect;
		tempPoint = (Point *)&globalWindRect;
		LocalToGlobal( tempPoint);
		globalWindRect.right += globalWindRect.left;
		globalWindRect.bottom += globalWindRect.top;
		
		rsrcRect = GetNamedResource( 'RECT', pictureName);
		if( rsrcRect)
		{
			HLock( rsrcRect);
			**((Rect **) rsrcRect) = globalWindRect;
			HUnlock( rsrcRect);
			ChangedResource( rsrcRect);
		}
		else		//	resource didn't load in
		{
			rsrcRect = NewHandle( sizeof(Rect));		//	8 bytes
			if( rsrcRect)
			{
				HLock( rsrcRect);
				**((Rect **) rsrcRect) = globalWindRect;
				HUnlock( rsrcRect);
				AddResource( rsrcRect, 'RECT', UniqueID( 'RECT'), pictureName);
			}
		}
		SetPort( savedPort);
	}
}


/*
 *
 *	This code is 68K specific ...
 */

#ifndef __powerc

pascal void DragHookForThreads( void)
{
	gDragRgnHandle = ReturnA3();
	
	GetPenState( &gDragPen);
	GetPort( &gDragPort);
	gSetWTitleOK = false;
	
	YieldToAnyThread();

	gDragRgnHandle = (RgnHandle) 0;
	gSetWTitleOK = true;
	SetPort( gDragPort);
	SetPenState( &gDragPen);
}

void	XDrawDragRgn( void)
{
	GrafPtr			savedPort;
	PenState		savedPen;
	short		  **firstRgnWord;
	
	if( gDragRgnHandle  &&  gDragRgnHandleValid)
	{
		GetPenState( &savedPen);
		GetPort( &savedPort);
		
		SetPort( gDragPort);
		SetPenState( &gDragPen);
		
		firstRgnWord = (short **)gDragRgnHandle;
		if( **firstRgnWord == 0x000A)
			FrameRect( (Rect *) &((*firstRgnWord)[1]));
		else
			PaintRgn( gDragRgnHandle);

		SetPort( savedPort);
		SetPenState( &savedPen);
	}
}

#else

/*
 *	This code is PowerPC filler ...
 */

pascal void DragHoodForThreads( void)
{
	//	empty PowerPC routines
}

pascal void XDrawDragRgn( void)
{

}

#endif
void	GWorldObj :: SetTitle( ConstStr255Param title)
{
	short			loop;
	
	if( gSetWTitleOK == true)
	{
		SetWTitle( me, title);
		needToUpdateWindowTitle = false;
	}
	else
		needToUpdateWindowTitle = true;
	
	loop = windowTitle[0] = title[0];
	while( loop)
	{
		windowTitle[loop] = title[loop];
		--loop;
	}
}
